% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [succinate_Matrix , coQ_Matrix , coQH2_Matrix , fumarate_Matrix , oxaloacetate_Matrix ]
%
%
% Free Parameters:
% [Vmf_SDH ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 succinate_Matrix
% 2 coQ_Matrix
% 3 coQH2_Matrix
% 4 fumarate_Matrix
% 5 oxaloacetate_Matrix
% 6 h_Matrix
% 7 m_Matrix
% 8 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
succinate_Matrix = x(1);
coQ_Matrix = x(2);
coQH2_Matrix = x(3);
fumarate_Matrix = x(4);
oxaloacetate_Matrix = x(5);
% Concentrations of H, Mg, and K
h_Matrix = x(6);
m_Matrix = x(7);
k_Matrix = x(8);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% succinate_Matrix
Kh(1) = 6.3607610505449683980985081666048586157558020204306e-06;
Km(1) = 0.063389622025238692715198851601599017158150672912598;
Kk(1) = 0.44392446615653530228584600081376265734434127807617;
% coQ_Matrix
Kh(2) = Inf;
Km(2) = Inf;
Kk(2) = Inf;
% coQH2_Matrix
Kh(3) = Inf;
Km(3) = Inf;
Kk(3) = Inf;
% fumarate_Matrix
Kh(4) = 9.73888149124482955035220221873260015854611992836e-05;
Km(4) = Inf;
Kk(4) = Inf;
% oxaloacetate_Matrix
Kh(5) = 0.00015083741503259175554402249375129940744955092668533;
Km(5) = 0.13709390976603802458910763562016654759645462036133;
Kk(5) = Inf;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);
P( 3 ) = 1  + h_Matrix/Kh(3) + m_Matrix/Km(3) + k_Matrix/Kk(3);
P( 4 ) = 1  + h_Matrix/Kh(4) + m_Matrix/Km(4) + k_Matrix/Kk(4);
P( 5 ) = 1  + h_Matrix/Kh(5) + m_Matrix/Km(5) + k_Matrix/Kk(5);

%% THERMODYNAMIC EQUATIONS
DGro_SDH =-1.35;

Keq_SDH_Matrix = exp(-DGro_SDH/RT)/P(1)/P(2)*P(3)*P(4);

%% FLUX EQUATIONS
%SDH_Matrix
a=succinate_Matrix;
b=coQ_Matrix;
p=coQH2_Matrix;
q=fumarate_Matrix;
KmA=467e-6;
KmB=480e-6;
KmP=2.45e-6;
KmQ=1200e-6;
Kia=120e-6;
Kiq=1275e-6;
KiOAA=1.5e-6;
KaSUC=450e-6;
KaFUM=375e-6;
Vmf_SDH=par(1);
Vmr=(Vmf_SDH*KmP*Kiq)/(Keq_SDH_Matrix*Kia*KmB);
alpha=(1+oxaloacetate_Matrix/KiOAA+a/KaSUC+q/KaFUM)/(1+a/KaSUC+q/KaFUM);
denom=Vmr*Kia*KmB*alpha+Vmr*KmB*a+Vmr*KmA*alpha*b+(Vmf_SDH*KmQ*alpha/Keq_SDH_Matrix)*p...
+(Vmf_SDH*KmP/Keq_SDH_Matrix)*q+Vmr*a*b+(Vmf_SDH*KmQ)/(Keq_SDH_Matrix*Kia)*a*p+(Vmr*KmA/Kiq)*b*q...
+(Vmf_SDH/Keq_SDH_Matrix)*p*q;
J_SDH_Matrix=(Vmf_SDH*Vmr*(a*b-p*q/Keq_SDH_Matrix))/denom;

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_SDH_Matrix ) / VWater_Matrix; % succinate_Matrix
f(2,:) = ( 0  - 1*J_SDH_Matrix ) / VWater_Matrix; % coQ_Matrix
f(3,:) = ( 0  + 1*J_SDH_Matrix ) / VWater_Matrix; % coQH2_Matrix
f(4,:) = ( 0  + 1*J_SDH_Matrix ) / VWater_Matrix; % fumarate_Matrix
f(5,:) = ( 0  ); % [clamped] % oxaloacetate_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2  3  4  5]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 0*J_SDH_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(6,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(7,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(8,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_SDH_Matrix];
